﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Analysis.AlgorithmBehavior.Analyzers;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AlgorithmBehaviorUnitTests {
  [TestClass]
  public class DistanceMatrixToPointsTest {
    [TestMethod]
    public void TestDistanceMatrixConversion() {
      int nrOfPoints = 4;
      int dim = 2;
      double[][] orgPoints = new double[nrOfPoints][];
      double[][] orgDm = new double[nrOfPoints][];
      double[][] newDm = new double[nrOfPoints][];
      double[][] newPoints = null;


      AllocArray(orgPoints, dim);
      AllocArray(orgDm, nrOfPoints);
      AllocArray(newDm, nrOfPoints);
      SamplePoints(orgPoints);
      CalculateDistanceMatrix(orgDm, orgPoints);

      newPoints = DistanceMatrixToPoints.ConvertDistanceMatrixToPoints(orgDm);

      CalculateDistanceMatrix(newDm, newPoints);
      //TODO

    }

    private static void SamplePoints(double[][] points) {
      Random rand = new Random();

      for (int i = 0; i < points.Length; i++) {
        for (int j = 0; j < points[i].Length; j++) {
          points[i][j] = rand.NextDouble() * 100;
        }
      }
    }

    private static void CalculateDistanceMatrix(double[][] dm, double[][] points) {
      for (int i = 0; i < points.Length; i++) {
        for (int j = 0; j < points.Length; j++) {
          dm[i][j] = CalcEuclideanDistance(points[i], points[j]);
        }
      }
    }

    private static double CalcEuclideanDistance(double[] p1, double[] p2) {
      double result = 0.0;
      for (int i = 0; i < p1.Length; i++) {
        result += Math.Pow(p1[i] - p2[i], 2);
      }
      return Math.Sqrt(result);
    }

    private static void AllocArray(double[][] arr, int size) {
      for (int i = 0; i < arr.Length; i++) {
        arr[i] = new double[size];
      }
    }
  }
}
